#! /usr/bin/env python
# -.- coding: utf-8 -.-

# Zeitgeist - Interactive prompt, aka "Joshua"
#
# Copyright © 2010 Siegfried-Angel Gevatter Pujals <rainct@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pwd
import time
import optparse

from zeitgeist import client, _config
from zeitgeist.datamodel import (Event, TimeRange, StorageState,ResultType,
	Interpretation, Manifestation)

class Parser:

	event_ids = None
	event_cache = []
	limit = 10
	prompt = ">"

	def __init__(self):
		self._iface = client.ZeitgeistDBusInterface()

	@staticmethod
	def _today_timestamp():
		now = time.localtime()
		return now.tm_hour * 3600 - now.tm_min * 60 - now.tm_sec

	def parse_line(self, line):
		line = line.strip(' \n\t!?.,;').lower().split()
		if len(line) == 1:
			if line[0] in ('bye', 'quit', 'cya', 'exit'):
				return False
			if line[0] in ('more', 'continue'):
				self.show_result()
				return True
		self.parse_query(line)
		return True
	
	def parse_query(self, query):
		time_range = TimeRange.always()		
		event_template = Event()
		result_type = ResultType.MostRecentSubjects
		if 'today' in query:
			time_range = TimeRange.from_seconds_ago(self._today_timestamp())
		self.event_ids = self._iface.FindEventIds(time_range, [event_template],
			StorageState.Any, self.limit * 3, result_type)
		self.event_cache = []
		self.show_result()
	
	def show_result(self):
		if not self.event_ids:
			if not self.event_cache:
				print "More what? You haven't asked me anything!"
			else:
				print "That's it, unless you have a more specific question!"
			return
		event_ids = self.event_ids[:self.limit]
		self.event_ids = self.event_ids[self.limit:]
		self.event_cache += [Event(e) for e in self._iface.GetEvents(event_ids)]
		offset = len(self.event_cache) - self.limit
		for i, event in enumerate(self.event_cache[offset:]):
			print " %d. %s" % (i + offset + 1, event.get_subjects()[0].text)
		self.prompt = "·>" if self.event_ids else ">"

def main():
	user = pwd.getpwuid(os.getuid())
	user = user.pw_gecos.split(",")[0] if user.pw_gecos else user.pw_name
	print "Greetings, %s." % user
	
	parser = Parser()
	status = True
	while status:
		try:
			line = raw_input("%s " % parser.prompt)
		except EOFError:
			status = False
			print
		else:
			if line:
				status = parser.parse_line(line)
	print "Bye."

if __name__ == "__main__":
	parser = optparse.OptionParser(version = _config.VERSION)
	options, arguments = parser.parse_args()
	main()
